--[[
    Author: Gaios
    USGN: 18271
    Version: 1.0
]]

_G.GAJOS_HPBARS = {}
local namespace = _G.GAJOS_HPBARS
local namespaceKey = 'GAJOS_HPBARS'

local hpbarPath = 'gfx/cs2dbr/hp/HP.png'
local coveredByFog = true

local USERS = {}
for id = 1, 32 do
    USERS[id] = {
        hpbarImgIds = {},
    }
end

local function removeByValue(tab, search)
    for i = 1, #tab do
        local val = tab[i]
        if val == search then
            table.remove(tab, i)
            return true
        end
    end
    return false
end

local function getHpbarFrameId(hp)
    if hp <= 30 then
        return 5
    elseif hp <= 47 then
        return 4
    elseif hp <= 64 then
        return 3
    elseif hp <= 81 then
        return 2
    end
    return 1
end

local function setHpbarFrame(userId, frameId)
    local user = USERS[userId]

    for _, imgId in pairs(user.hpbarImgIds) do
        imageframe(imgId, frameId)
    end
end

local function removeHpbarImages(userId)
    local user = USERS[userId]

    for _, imgId in pairs(user.hpbarImgIds) do
        freeimage(imgId)
        table.remove(user.hpbarImgIds, _)
    end
end

local function addHpbarImages(userId)
    removeHpbarImages(userId)

    local user = USERS[userId]
    local teamId = player(userId, 'team')

    if teamId == 0 then
        return false
    end

    local teammateIds = player(0, 'team'..teamId)
    removeByValue(teammateIds, userId)

    for _, teammateId in pairs(teammateIds) do
        local skipFog = coveredByFog and 0 or 1
        local imgId = image('<spritesheet:'..hpbarPath..':32:64:n>', 0, skipFog, userId + 200, teammateId)
        table.insert(user.hpbarImgIds, imgId)
    end

    local hp = player(userId, 'health')
    local frameId = getHpbarFrameId(hp)
    setHpbarFrame(userId, frameId)

    return true
end

local function resetHpbarImages(userId)
    removeHpbarImages(userId)
    addHpbarImages(userId)
end

local function removeImages()
    for userId = 1, 32 do
        removeHpbarImages(userId)
    end
end

function namespace.onStartround(mode)
    removeImages()

    local lists = {player(0, 'team1'), player(0, 'team2')}

    for _, list in pairs(lists) do
        for _, userId in pairs(list) do
            addHpbarImages(userId)
        end
    end
end

function namespace.onJoin(userId)
    resetHpbarImages(userId)
end

function namespace.onHit(userId, sourceId, wpn, hpdmg)
    local user = USERS[userId]
    local hp = player(userId, 'health')
    local frameId = getHpbarFrameId(hp - hpdmg)

    setHpbarFrame(userId, frameId)
end

function namespace.onSpawn(userId)
    resetHpbarImages(userId)
end

addhook('startround', namespaceKey..'.onStartround')
addhook('join', namespaceKey..'.onJoin')
addhook('hit', namespaceKey..'.onHit')
addhook('spawn', namespaceKey..'.onSpawn')
